class com.cintegrity.Lib.Exception.SimpleExceptionStackImpl:
  define private variable cin_CurrentExceptionSequence as integer no-undo initial 0.
  define private variable clg_InDebug as logical no-undo initial no.
  define private variable cch_SeverityName as character no-undo initial "ERROR,WARNING,CAUTION,NOTE,DEBUG".
  define private variable cch_Language as character no-undo initial "ENU".
  define private temp-table ctt_ExceptionStack no-undo
      field in_Sequence as integer
      field in_ExceptionNo as integer
      field in_Severity as integer
      field ch_Message as character
      field ch_ProgramName as character
      index ix_BySequence as primary unique in_Sequence descending
      index ix_BySeverity in_Severity
      .
  define private variable cin_NumReferences as integer no-undo.
  define private variable cch_LineBreak as character no-undo.
  /* ----------------------------------------------------------------------- */
  constructor public SimpleExceptionStackImpl():
    cch_LineBreak = chr(1).
  end constructor.
  /* ----------------------------------------------------------------------- */
  destructor public SimpleExceptionStackImpl():
    empty temp-table ctt_ExceptionStack.
  end.
  /* ----------------------------------------------------------------------- */
  method public void pushException(
      input imin_ExceptionNumber as integer,
      input imin_ExceptionSeverity as integer,
      input imch_Arguments as character
      ):

    create ctt_ExceptionStack.
    assign
        cin_CurrentExceptionSequence = cin_CurrentExceptionSequence + 1
        ctt_ExceptionStack.in_Sequence = cin_CurrentExceptionSequence
        ctt_ExceptionStack.in_ExceptionNo = imin_ExceptionNumber
        ctt_ExceptionStack.in_Severity = imin_ExceptionSeverity
        ctt_ExceptionStack.ch_Message = imch_Arguments
        .
    if num-entries(program-name(3)," ") gt 1
    then ctt_ExceptionStack.ch_ProgramName = entry(1,program-name(3)," ") + " of " + entry(2,program-name(3)," ").
    else ctt_ExceptionStack.ch_ProgramName = program-name(3).
  end method.
  /* ----------------------------------------------------------------------- */
  method public void trace(
      input imch_TraceMessage as character
      ):
    create ctt_ExceptionStack.
    assign
        cin_CurrentExceptionSequence = cin_CurrentExceptionSequence + 1
        ctt_ExceptionStack.in_Sequence = cin_CurrentExceptionSequence
        ctt_ExceptionStack.in_ExceptionNo = -1
        ctt_ExceptionStack.in_Severity = 5
        ctt_ExceptionStack.ch_Message = imch_TraceMessage
        .
    if num-entries(program-name(3)," ") gt 1
    then ctt_ExceptionStack.ch_ProgramName = entry(1,program-name(3)," ") + " of " + entry(2,program-name(3)," ").
    else ctt_ExceptionStack.ch_ProgramName = program-name(3).
  end method.
  /* ----------------------------------------------------------------------- */
  method public character popException(
      input imin_MinSeverity as integer
      ):
    define variable mch_Message as character no-undo.

    for each ctt_ExceptionStack
        where ctt_ExceptionStack.in_Severity LE imin_MinSeverity
        by ctt_ExceptionStack.in_Sequence descending
        :
      leave.
    end.
    if available( ctt_ExceptionStack )
    then do:
      mch_Message = formatStandardMessage( ctt_ExceptionStack.in_Sequence ).
      delete ctt_ExceptionStack.
      return mch_Message.
    end.
    else do:
      return ?.
    end.
  end method.
  /* ----------------------------------------------------------------------- */
  method public void dropException(
      input imin_MinSeverity as integer
      ):
    for each ctt_ExceptionStack
        where ctt_ExceptionStack.in_Severity LE imin_MinSeverity
        by ctt_ExceptionStack.in_Sequence descending
        :
      leave.
    end.
    if available( ctt_ExceptionStack )
    then delete ctt_ExceptionStack.
  end method.
  /* ----------------------------------------------------------------------- */
  method public integer getExceptionNo(
      input imin_MinSeverity as integer,
      input imin_Count as integer
      ):
    define variable min_Count as integer no-undo.
    for each ctt_ExceptionStack
        where ctt_ExceptionStack.in_Severity LE imin_MinSeverity
        by ctt_ExceptionStack.in_Sequence descending
        :
      min_Count = min_Count + 1.
      if min_Count EQ imin_Count then leave.
    end.
    if available( ctt_ExceptionStack )
    then return ctt_ExceptionStack.in_ExceptionNo.
    else return ?.
  end method.
  /* ----------------------------------------------------------------------- */
  method public integer numExceptions(
      input imin_MinSeverity as integer
      ):
    define variable min_Count as integer no-undo.
    for each ctt_ExceptionStack
        where ctt_ExceptionStack.in_Severity LE imin_MinSeverity
        by ctt_ExceptionStack.in_Sequence descending
        :
      min_Count = min_Count + 1.
    end.
    return min_Count.
  end method.
  /* ----------------------------------------------------------------------- */
  method public void clearExceptions():
    empty temp-table ctt_ExceptionStack.
    cin_CurrentExceptionSequence = 0.
  end method.
  /* ----------------------------------------------------------------------- */
  method public void setDebug():
    clg_InDebug = true.
  end method.
  /* ----------------------------------------------------------------------- */
  method public void unsetDebug():
    clg_InDebug = false.
  end method.
  /* ----------------------------------------------------------------------- */
  method public logical inDebug():
    return clg_InDebug.
  end method.
  /* ------------------------------------------------------------------------- */
  method public longchar toLongString(
      input imin_Severity as integer
      ):
    define variable mlc_ExceptionList as longchar no-undo.
    define variable mch_Message as character no-undo.

    for each ctt_ExceptionStack no-lock
        where ctt_ExceptionStack.in_Severity LE imin_Severity
        by ctt_ExceptionStack.in_Sequence descending
        :
      mch_Message = formatStandardMessage( ctt_ExceptionStack.in_Sequence ).
      mlc_ExceptionList = mlc_ExceptionList + mch_Message + cch_LineBreak.
    end.
    return mlc_ExceptionList.
  end method.
  /* ----------------------------------------------------------------------- */
  method public longchar writeExternal():
    define variable mlc_Serialized as longchar no-undo.
    mlc_Serialized = "<ExceptionStackImpl>".
    mlc_Serialized = mlc_Serialized + "<cin_CurrentExceptionSequence>" + string(cin_CurrentExceptionSequence) + "</cin_CurrentExceptionSequence>".
    mlc_Serialized = mlc_Serialized + "<clg_InDebug>" + string(clg_InDebug) + "</clg_InDebug>".
    mlc_Serialized = mlc_Serialized + "<cch_Language>" + cch_Language + "</cch_Language>".
    mlc_Serialized = mlc_Serialized + "<ctt_ExceptionStack>".
    for each ctt_ExceptionStack by ctt_ExceptionStack.in_Sequence:
      mlc_Serialized = mlc_Serialized +
          "<ctt_ExceptionStackRow>" +
          "<in_Sequence>" + string(ctt_ExceptionStack.in_Sequence) + "</in_Sequence>" +
          "<in_ExceptionNo>" + string(ctt_ExceptionStack.in_ExceptionNo) + "</in_ExceptionNo>" +
          "<in_Severity>" + string(ctt_ExceptionStack.in_Severity) + "</in_Severity>" +
          "<ch_Message>" + string(ctt_ExceptionStack.ch_Message) + "</ch_Message>" +
          "<ch_ProgramName>" + string(ctt_ExceptionStack.ch_ProgramName) + "</ch_ProgramName>" +
          "</ctt_ExceptionStackRow>"
          .
    end.
    mlc_Serialized = mlc_Serialized + "</ctt_ExceptionStack>".
    mlc_Serialized = mlc_Serialized + "</ExceptionStackImpl>".
    return mlc_Serialized.
  end method.
  /* ----------------------------------------------------------------------- */
  method private character formatStandardMessage(
      input imin_Sequence as integer
      ):
    define variable mch_Severity as character no-undo.
    define variable mch_Message as character no-undo.

    find ctt_ExceptionStack
        where ctt_ExceptionStack.in_Sequence EQ imin_Sequence
        no-error.
    if available( ctt_ExceptionStack )
    then do:
      if      ctt_ExceptionStack.in_Severity ge 1
          and ctt_ExceptionStack.in_Severity le 5
      then mch_Severity = entry( ctt_ExceptionStack.in_Severity, cch_SeverityName ).
      else mch_Severity = "UNKNOWN".
      mch_Message = substring(mch_Severity + ":        ",1,9) + ctt_ExceptionStack.ch_Message.
      if ctt_ExceptionStack.in_ExceptionNo gt 0
      then mch_Message = mch_Message + " (" + string(ctt_ExceptionStack.in_ExceptionNo) + ")".
      if ctt_ExceptionStack.in_ExceptionNo EQ -1
      then mch_Message = mch_Message + " [TRACE]".
      if clg_InDebug
      then mch_Message = mch_Message +  " from " + ctt_ExceptionStack.ch_ProgramName.
      return mch_Message.
    end.
    else do:
      return ?.
    end.
  end method.
  /* -------- METHODS FOR PSEUDO-SINGLETON --------------------------------- */
  method public void addReference():
    cin_NumReferences = cin_NumReferences + 1.
  end method.
  /* ----------------------------------------------------------------------- */
  method public void dropReference():
    cin_NumReferences = max(cin_NumReferences - 1, 0 ).
  end method.
  /* ----------------------------------------------------------------------- */
  method public integer numReferences():
    return cin_NumReferences.
  end method.
  /* ----------------------------------------------------------------------- */
end class.
